-- Scenario globals are console globals, some handy-dandies are here too

local ename = {} for k,v in next,defines.events do ename[v]=k end
--
local notrace={}
keep_resources = (settings.startup['benchmark-scenario-keep-resources'] or {}).value
local starttick = 0
local chunksrem = 0
--
function default_trace(ev) print(ename[ev.name],serpent.line(ev)) end
local bootstraps={init=1,load=1,configuration_changed=1}
function on (s,f)
    local on_s ='on_'..s
    local e = defines.events[on_s]
    if e then
	notrace[e]=true
	return script.on_event(e,f)
    elseif bootstraps[s] then
	return script[on_s](f)
    else -- a custom input
	return script.on_event(s,f)
	end
    end
mfd='marked_for_deconstruction'
function iod(text) -- "instead of deconstructing"
    return load([[return
	function(ev) local e,p = ev.entity, game.players[ev.player_index]
	    local b1,b2 = e.get_inventory(1),e.get_inventory(2)
	    e.cancel_deconstruction(p.force)
	    ]] .. text .. ' end'
	)()
    end
--     /c on(mfd,iod([[ e.insert{name='iron-plate',count=1e9} ]]))
--     /c on(mfd,iod([[ return b2 and b2.clear() ]])) -- clears assembler input

-- syntax sugar & other dubious sweeteners
local function selfmeta(t) return setmetatable(t,t) end
local min,max = math.min, math.max
local floor,ceil = math.floor, math.ceil
local clamp_range=function(n,v,x) return v<n and n or x<v and x or v end

--[[ runtime setting too distracting outside the scenario, made startup
on('runtime_mod_setting_changed',function(ev)
    if ev.setting=='benchmark-scenario-keep-resources' then
	keep_resources = settings.global['benchmark-scenario-keep-resources'].value
	-- game.print(serpent.line(settings.global['benchmark-scenario-keep-resources']))
	end
    end)
--]]

on('load',function()
    global.reload()
    end)
on('init',function()
    function global.reload() for k,v in next,global do _G[k]=v end end
    global.on=on
    global.iod=iod
    global.mfd=mfd
    global.reload()
    keep_resources = (settings.startup['benchmark-scenario-keep-resources']or {}).value
    game.forces.player.research_all_technologies()
    game.forces.player.enable_all_recipes()
    game.forces.player.character_running_speed_modifier=1.25
    game.forces.player.character_reach_distance_bonus=64
    game.forces.player.character_build_distance_bonus=64
    game.forces.player.character_inventory_slots_bonus=40
    game.surfaces[1].always_day=true
    game.surfaces[1].request_to_generate_chunks({0,0},32) -- to avoid later benchmark interference
    chunksrem = 65*65
    starttick = game.tick
    on('tick',function()
	if game.tick == starttick then
	    game.print('the game will chug for a few seconds, pregenerating'..
		' 65×65 chunks to avoid later benchmark interference')
	elseif game.tick == starttick+120 then
	    game.print('chugging part should be about done, background repaving of the outlands in progress...')
	    on('tick',nil)
	    on('research_started',function(e)
		-- one-tick delay to keep the auto-research mod happy
		local fn,tn,t=e.research.force.name,e.research.name,e.tick
		on('tick',function(...) if game.tick>t then
		    on('tick',nil)
		    local _=game.forces[fn].technologies[tn]
		    if _.researched then _.level = _.level+1 end
		    _.researched=true
		    end end)
		end)
	    end
	end)
    end)

on('player_created',function(e)
    local p=game.players[e.player_index]
    p.cheat_mode = true
    local count = false
    for _,v in next,{1,'power-armor-mk2' ,'steel-axe'
    ,1000,'rail','train-stop','rail-signal'
    ,10,'locomotive',30,'cargo-wagon'
    ,'small-electric-pole','medium-electric-pole','big-electric-pole'
    ,'substation',1,'electric-energy-interface'
    ,'arithmetic-combinator','decider-combinator','constant-combinator'
    ,'red-wire','green-wire'
    ,'construction-robot'
    ,'stack-inserter','stack-filter-inserter'
    ,'express-transport-belt','express-underground-belt','express-splitter'
    ,'steel-chest'
    ,'logistic-chest-requester','logistic-chest-active-provider'
    ,'logistic-chest-passive-provider', 'logistic-chest-storage'
    ,1,'submachine-gun','uranium-rounds-magazine'
    ,'rail-chain-signal'
    } do
        if type(v) == 'number' then
            count = v
        elseif not game.item_prototypes[v] then
	    p.print(string.format('no %s prototype?',v))
	    count=false
	else
	    p.insert{name=v, count=count or game.item_prototypes[v].stack_size}
            count=false
            end
        end

    local grid=p.get_inventory(defines.inventory.player_armor)[1].grid
    for _,v in next,{4,'fusion-reactor-equipment'
            ,18,'battery-mk2-equipment'
            ,4,'personal-roboport-mk2-equipment'
	    ,8,'energy-shield-mk2-equipment'} do
        if type(v) == 'number' then
            count = v
        elseif not game.item_prototypes[v] then
	    p.print(string.format('no %s prototype?',v))
	    count=false
	else
	    if not count then
		count=game.item_prototypes[v].stack_size
		end
	    for _=1,count do
		local e=grid.put{name=v}
		if e then
		    count = count-1
		    if string.sub(v,1,7)=='battery' then
			e.energy = e.max_energy
			end
		    end
		end
	    if count>0 then p.insert{name=v,count=count} end
	    count=false
	    end
	end
    end)

local pkey = function(x,y) return x+y/2^21 end
local markers={} for x = -301,299,300 do for y=-301,299,300 do
    markers[pkey(x,y)] = 'hazard-concrete-left'
    markers[pkey(x+1,y)] = 'hazard-concrete-right'
    markers[pkey(x,y+1)] = 'hazard-concrete-right'
    markers[pkey(x+1,y+1)] = 'hazard-concrete-left'
    end end
local tiles={} for n=1,32*32 do tiles[n]={name='',position={x=0,y=0}} end
--
on('chunk_generated', function(ev)
    for _, entity in next,ev.surface.find_entities(ev.area) or {} do
	if entity.valid and not ( entity.type == 'player'
	   or keep_resources and entity.type == 'resource' ) then
	    entity.destroy()
	    end
        end
    local x,y = ev.area.left_top.x, ev.area.left_top.y
    local n = 1
    for x = x-1, x + 30 do
        for y = y-1, y + 30 do
            tiles[n].name = markers[pkey(x,y)] or 'lab-dark-2'
	    tiles[n].position.x = x
	    tiles[n].position.y = y
	    n = n+1
            end
        end
    ev.surface.set_tiles(tiles,false)
    chunksrem = chunksrem - 1
    if chunksrem == 0 then
	game.forces.player.chart_all()
	game.print('background repaving complete!'..
	    ' Save now and nothing will interfere with benchmarking')
    elseif chunksrem < 0 then
	print('chunk',serpent.line(ev.area),'generated')
	end
    end)
-- vim: set sts=4 sw=4 tw=71 lbr fo=tcroq com=\:-- :
